//= require <external/IconCombo>

Ext.namespace("dSS", "dSS.config");

// form *must* react to the submit event and fire a formsubmitted
// event in response; it *must* also fire a formloaded event once all d
// has been received
dSS.BasicGroupForm = Ext.extend(Ext.form.FormPanel, {
    constructor: function(config) {

        this.groupName = "";
        this.groupID = config.groupID;

        if (config.groupColor !== undefined) {
            this.groupColor = config.groupColor;
        }

        if (config.groupName !== undefined) {
            this.groupName = config.groupName;
        }

        if (this.groupColor === undefined) {
            this.groupColor = dSS.config.groups.LIGHT;
        }

        this.uCss = 'UserGroup';

        var list =
        [
            [
                dSS.config.groups.LIGHT,
                dSS.config.GroupColorIconCls[dSS.config.groups.LIGHT] +
                                            this.uCss,
                dSS._(dSS.config.groupNames[dSS.config.groups.LIGHT])
            ],
            [
                dSS.config.groups.SHADOW,
                dSS.config.GroupColorIconCls[dSS.config.groups.SHADOW] +
                                            this.uCss,
                dSS._(dSS.config.groupNames[dSS.config.groups.SHADOW])
            ],
            [
                dSS.config.groups.HEATING,
                dSS.config.GroupColorIconCls[dSS.config.groups.HEATING] +
                                            this.uCss,
                dSS._(dSS.config.groupNames[dSS.config.groups.HEATING])
            ],
            [
                dSS.config.groups.COOLING,
                dSS.config.GroupColorIconCls[dSS.config.groups.COOLING] +
                                            this.uCss,
                dSS._(dSS.config.groupNames[dSS.config.groups.COOLING])
            ],
            [
                dSS.config.groups.VENTILATION,
                dSS.config.GroupColorIconCls[dSS.config.groups.VENTILATION] +
                                            this.uCss,
                dSS._(dSS.config.groupNames[dSS.config.groups.VENTILATION])
            ],
            [
                dSS.config.groups.AUDIO,
                dSS.config.GroupColorIconCls[dSS.config.groups.AUDIO] +
                                            this.uCss,
                dSS._(dSS.config.groupNames[dSS.config.groups.AUDIO])
            ],
            [
                dSS.config.groups.VIDEO,
                dSS.config.GroupColorIconCls[dSS.config.groups.VIDEO] +
                                            this.uCss,
                dSS._(dSS.config.groupNames[dSS.config.groups.VIDEO])
            ]
        ];

        this.addEvents('formsubmitted', 'formloaded', 'formchanged',
                       'formretry');

        this.colorSelection = new Ext.ux.IconCombo({
            typeAhead: false,
            triggerAction: 'all',
            lazyRender:true,
            forceSelection: true,
            editable: false,
            disabled: (config.fixedColor === true),
            width: 150,
            mode: 'local',
            fieldLabel: dSS._("Color group"),
            tpl: new Ext.XTemplate(
                '<tpl for="."><div class="x-combo-list-item">',
                '<table><tbody><tr>',
                '<td>',
                '<div class="{css} x-icon-combo-icon"></div></td>',
                '<td>{displayText}</td>',
                '</tr></tbody></table>',
                '</div></tpl>'),
            store: new Ext.data.ArrayStore({
                id: 0,
                fields:
                [
                    'id',
                    'css',
                    'displayText'
                ],
                data: list
            }),
            valueField: 'id',
            displayField: 'displayText',
            iconClsField: 'css'
        });

        this.groupNameField = new Ext.form.TextField({
            fieldLabel: dSS._("Group name"),
            emptyText: dSS._("Enter group name..."),
            allowBlank: false,
            width: 150
        });

        Ext.apply(this, {
            bubbleEvents:
            [
                'formsubmitted', 'formloaded', 'formchanged', 'formretry'
            ],
            padding: 5,
            height: 70,
            width: 270,
            scope: this,
            layout: 'fit',
            items:
            [
                {
                    xtype: 'panel',
                    border: false,
                    layout: 'form',
                    items:
                    [
                        {
                            xtype: 'box',
                            height: 5
                        },
                        this.groupNameField,
                        this.colorSelection
                    ]
                }
            ]
        });

        dSS.BasicGroupForm.superclass.constructor.apply(this, arguments);

        this.colorSelection.on('afterrender', function() {
                if ((this.closed !== true) &&
                   (((this.groupColor >= dSS.config.groups.LIGHT) &&
                     (this.groupColor < dSS.config.groups.JOKER)) ||
                    (this.groupColor == dSS.config.groups.COOLING) ||
                    (this.groupColor == dSS.config.groups.VENTILATION))) {
                    this.colorSelection.setValue(this.groupColor);
                }
                this.colorSelection.on('select', function() {
                    if (this.groupNameField.isValid()) {
                        this.fireEvent('formchanged');
                    }
                },
                this);
            },
            this);

         this.groupNameField.on('afterrender', function() {
                if (this.closed !== true) {
                    this.groupNameField.setValue(this.groupName);
                    this.groupNameField.on('valid', function() {
                        this.fireEvent('formchanged');
                    },
                    this);
                }
            },
            this);

        this.on('beforedestroy', function() {
                this.closed = true;
            },
            this);
    }
});

dSS.AddGroupForm = Ext.extend(dSS.BasicGroupForm, {
    initComponent: function() {
        dSS.AddGroupForm.superclass.initComponent.apply(this, arguments);

        this.on('submit', function() {
            if (this.groupNameField.isValid() !== true) {
                Ext.Msg.alert(
                    dSS._("Error"),
                    dSS._("Please enter a valid group name!"),
                    this.fireEvent('formretry'));
                return;
            }

            var color = this.colorSelection.getValue();
            var name = this.groupNameField.getValue();

            Ext.Ajax.request({
                url: '/json/structure/addGroup',
                method: "GET",
                params:
                {
                    zoneID: 0, // global group
                    groupColor: color,
                    groupAutoSelect: "global",
                    groupName: name
                },
                scope: this,
                success: function(result, request) {
                    try {
                        var json = Ext.util.JSON.decode(result.responseText);
                        if (!json.ok) {
                            Ext.Msg.alert(dSS._("Error"),
                                dSS._("Could not add new group") + ": " +
                                json.message);
                            this.fireEvent('formsubmitted');
                            return;
                        }
                        var groupStore = dSS.data.Loader.getGroupStore();
                        var record = {
                            name: name,
                            id: json.result.groupID,
                            sortorder: 200,
                            color: color,
                            css: dSS.config.GroupColorIconCls[color] +
                                                              this.uCss
                        };
                        groupStore.loadData({ groups: [ record ] }, true);
                        dSS.MessageBus.fireEvent('statusmessage',
                                dSS._("Group") + ' #' + json.result.groupID +
                                ' ' + dSS._("was created"));
                    } catch (err) {
                        Ext.Msg.alert(dSS._("Error"),
                            dSS._("Could not add new group") + ": " + err);
                    }
                    this.fireEvent('formsubmitted');
                },
                failure: function(result, request) {
                    var message = '';
                    try {
                        var json = Ext.util.JSON.decode(result.responseText);
                        if ((!json.ok) && (json.message.length > 0)) {
                            message = ': ' + json.message;
                        }
                    } catch(err) {}
                    Ext.Msg.alert(dSS._("Error"),
                                  dSS._("Could not create new group") +
                                  message);

                    this.fireEvent('formsubmitted');
                }
            });
        },
        this);
    }
});


Ext.reg('dssaddgroupform', dSS.AddGroupForm);

dSS.EditGroupForm = Ext.extend(dSS.BasicGroupForm, {
    initComponent: function() {

        dSS.EditGroupForm.superclass.initComponent.apply(this, arguments);

        this.on('submit', function() {
            if (this.groupNameField.isValid() !== true) {
                Ext.Msg.alert(
                    dSS._("Error"),
                    dSS._("Please enter a valid group name!"),
                    this.fireEvent('formretry'));
                return;
            }

            var color = this.colorSelection.getValue();
            var name = this.groupNameField.getValue();
            var delay = 1000;
            var sendEvent = true;
            if (name !== this.groupName) {
                this.setGroupName(name);
                sendEvent = false;
            } else {
                delay = 0;
            }
            if (color !== this.groupColor) {
                this.setGroupColor.defer(delay, this, [ color, sendEvent ]);
            }
        },
        this);
    },
    setGroupName: function(name) {
        Ext.Ajax.request({
            url: '/json/structure/groupSetName',
            method: "GET",
            params:
            {
                zoneID: 0, // global group
                groupID: this.groupID,
                newName: name
            },
            scope: this,
            success: function(result, request) {
                try {
                    var json = Ext.util.JSON.decode(result.responseText);
                    if (!json.ok) {
                        Ext.Msg.alert(dSS._("Error"),
                            dSS._("Could not set group name") + ": " +
                            json.message);
                        this.fireEvent('formsubmitted');
                        return;
                    }
                    var groupStore = dSS.data.Loader.getGroupStore();
                    var record = groupStore.getById(this.groupID);
                    if (record !== undefined) {
                        record.set('name', name);
                        record.commit();
                        groupStore.doSort();
                    }
                    dSS.MessageBus.fireEvent('statusmessage',
                            dSS._("Group") + ' #' + this.groupID +
                            ' ' + dSS._("was renamed"));
                } catch (err) {
                    Ext.Msg.alert(dSS._("Error"),
                        dSS._("Could not rename group") + ": " + err);
                }
                this.fireEvent('formsubmitted');
            },
            failure: function(result, request) {
                var message = '';
                try {
                    var json = Ext.util.JSON.decode(result.responseText);
                    if ((!json.ok) && (json.message.length > 0)) {
                        message = ': ' + json.message;
                    }
                } catch(err) {}
                Ext.Msg.alert(dSS._("Error"),
                              dSS._("Could not rename group") +
                              message);

                this.fireEvent('formsubmitted');
            }
        });
    },
    setGroupColor: function(color, sendEvent) {
        Ext.Ajax.request({
            url: '/json/structure/groupSetColor',
            method: "GET",
            params:
            {
                zoneID: 0, // global group
                groupID: this.groupID,
                newColor: color
            },
            scope: this,
            success: function(result, request) {
                try {
                    var json = Ext.util.JSON.decode(result.responseText);
                    if (!json.ok) {
                        Ext.Msg.alert(dSS._("Error"),
                            dSS._("Could not set group color") + ": " +
                            json.message);
                        if (sendEvent === true) {
                            this.fireEvent('formsubmitted');
                        }
                        return;
                    }
                    var groupStore = dSS.data.Loader.getGroupStore();
                    var record = groupStore.getById(this.groupID);
                    if (record !== undefined) {
                        record.set('color', color);
                        record.set('css', dSS.config.GroupColorIconCls[color] +
                                          this.uCss);
                        record.commit();
                        groupStore.doSort();
                    }
                    dSS.MessageBus.fireEvent('statusmessage',
                            dSS._("Group") + ' #' + this.groupID +
                            ' ' + dSS._("color has been changed"));
                } catch (err) {
                    Ext.Msg.alert(dSS._("Error"),
                        dSS._("Could not set group color") + ": " + err);
                }
                if (sendEvent === true) {
                    this.fireEvent('formsubmitted');
                }
            },
            failure: function(result, request) {
                var message = '';
                try {
                    var json = Ext.util.JSON.decode(result.responseText);
                    if ((!json.ok) && (json.message.length > 0)) {
                        message = ': ' + json.message;
                    }
                } catch(err) {}
                Ext.Msg.alert(dSS._("Error"),
                              dSS._("Could not set group color") +
                              message);

                if (sendEvent === true) {
                    this.fireEvent('formsubmitted');
                }
            }
        });
    }
});


Ext.reg('dsseditgroupform', dSS.EditGroupForm);
